from django.db import models
from django.utils.translation import gettext_lazy as _


class LogFile(models.Model):
    name = models.CharField(_('name'), max_length=180)
    path = models.CharField(_('path'), max_length=500)

    def __unicode__(self):
        return '%s' % self.name

    class Meta:
        verbose_name = _('Log file')
        verbose_name_plural = _('Log files')


class Filter(models.Model):
    name = models.CharField(_('name'), max_length=180)
    regex = models.CharField(_('regex'), max_length=500)

    def __unicode__(self):
        return '%s | %s: %s ' % (self.name, _('pattern'), self.regex)

    class Meta:
        verbose_name = _('filter')
        verbose_name_plural = _('filters')


class LogsClipboard(models.Model):
    name = models.CharField(_('name'), max_length=180)
    notes = models.TextField(_('notes'), blank=True, null=True)
    logs = models.TextField(_('logs'))
    log_file = models.ForeignKey(LogFile, on_delete=models.CASCADE, verbose_name=_('log file'))

    def __unicode__(self):
        return "%s" % self.name

    class Meta:
        verbose_name = _('logs clipboard')
        verbose_name_plural = _('logs clipboard')



import os
import json
from django.http import HttpResponse
from django.template import RequestContext
from django.shortcuts import render
from logtailer.models import LogsClipboard, LogFile
from django.utils.translation import gettext as _
from django.views.decorators.csrf import csrf_exempt
from django.contrib.admin.views.decorators import staff_member_required


@staff_member_required
def read_logs(request):
    context = {}
    return render(request, 'logtailer/log_reader.html',
                  context, RequestContext(request, {}),)


def get_history(f, lines=0):
    buffer_size = 1024
    f.seek(0, os.SEEK_END)
    bytes = f.tell()
    size = lines
    block = -1
    data = []
    while size > 0 and bytes > 0:
        if bytes - buffer_size > 0:
            # Seek back one whole buffer_size
            f.seek(f.tell()+block*buffer_size, 0)
            # read buffer
            data.append(f.read(buffer_size))
        else:
            # file too small, start from beginning
            f.seek(0, 0)
            # only read what was not read
            data.append(f.read(bytes))
        lines_found = data[-1].count('\n')
        size -= lines_found
        bytes += block*buffer_size
        block -= 1
    return ''.join(data).splitlines(True)[-lines:]


@staff_member_required
def get_log_lines(request, file_id):
    history = int(request.GET.get('history', 0))
    try:
        file_record = LogFile.objects.get(id=file_id)
    except LogFile.DoesNotExist:
        return HttpResponse(json.dumps([_('error_logfile_notexist')]),
                            content_type='text/html')
    content = []
    file = open(file_record.path, 'r')

    if history > 0:
        content = get_history(file, history)
        content = [line.replace('\n','<br/>') for line in content]
    else:
        last_position = request.session.get('file_position_%s' % file_id)
        file.seek(0, os.SEEK_END)
        if last_position and last_position <= file.tell():
            file.seek(last_position)

        for line in file:
            content.append('%s' % line.replace('\n','<br/>'))

    request.session['file_position_%s' % file_id] = file.tell()
    file.close()
    return HttpResponse(json.dumps(content), content_type='application/json')


@staff_member_required
def save_to_clipoard(request):
    LogsClipboard(name=request.POST['name'],
                  notes=request.POST['notes'],
                  logs=request.POST['logs'],
                  log_file=LogFile.objects.get(id=int(request.POST['file']))).save()
    return HttpResponse(_('loglines_saved'), content_type='text/html')



# Generated by Django 3.2.4 on 2021-08-31 05:38

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('logtailer', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='logfile',
            options={'verbose_name': 'Log file', 'verbose_name_plural': 'Log files'},
        ),
        migrations.AlterModelOptions(
            name='logsclipboard',
            options={'verbose_name': 'logs clipboard', 'verbose_name_plural': 'logs clipboard'},
        ),
        migrations.AlterField(
            model_name='logsclipboard',
            name='log_file',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='logtailer.logfile', verbose_name='log file'),
        ),
    ]



from django import template
from logtailer.models import Filter

register = template.Library()


@register.inclusion_tag('logtailer/templatetags/filters.html')
def filters_select():
    filters = Filter.objects.all()
    return {'filters': filters}


# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-07-25 19:42
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Filter',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=180, verbose_name='name')),
                ('regex', models.CharField(max_length=500, verbose_name='regex')),
            ],
            options={
                'verbose_name': 'filter',
                'verbose_name_plural': 'filters',
            },
        ),
        migrations.CreateModel(
            name='LogFile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=180, verbose_name='name')),
                ('path', models.CharField(max_length=500, verbose_name='path')),
            ],
            options={
                'verbose_name': 'log_file',
                'verbose_name_plural': 'log_files',
            },
        ),
        migrations.CreateModel(
            name='LogsClipboard',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=180, verbose_name='name')),
                ('notes', models.TextField(blank=True, null=True, verbose_name='notes')),
                ('logs', models.TextField(verbose_name='logs')),
                ('log_file', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='logtailer.LogFile', verbose_name='log_file')),
            ],
            options={
                'verbose_name': 'logs_clipboard',
                'verbose_name_plural': 'logs_clipboard',
            },
        ),
    ]



from django.urls import path
from . import views

urlpatterns = [
    path('readlogs/', views.read_logs, name='logtailer_read_logs'),
    path('get-log-line/<int:file_id>/', views.get_log_lines, name='logtailer_get_log_lines'),
    path('save-to-clipboard/', views.save_to_clipoard, name="logtailer_save_to_clipboard"),
]



from django.contrib import admin
from django.utils.translation import gettext as _
from django.http import HttpResponse, HttpResponseRedirect
from django.urls import path, reverse
from logtailer.models import LogFile, Filter, LogsClipboard


class LogFileAdmin(admin.ModelAdmin):
    list_display = ('__unicode__', 'path')

    class Media:
        css = {
            'all': ('logtailer/css/logtailer.css',)
        }

    def get_urls(self):
        info = self.model._meta.app_label, self.model._meta.model_name
        urls = super().get_urls()
        my_urls = [
            path('<int:object_id>/download/',
                self.admin_site.admin_view(self.download), {},
                name="%s_%s_download" % info),
        ]
        return my_urls + urls

    def download(self, request, object_id):
        try:
            log_file = self.get_object(request, object_id)
            with open(log_file.path, 'r') as f:
                buffer = f.read()
            response = HttpResponse(buffer, content_type='plain/text')
            response['Content-Disposition'] = 'attachment; filename=%s' % log_file.name
        except Exception as e:
            try:
                from django.contrib import messages
                self.message_user(request, _('ERROR') + ': ' + str(e), level=messages.ERROR)
            except:
                pass
            response = HttpResponseRedirect(reverse('admin:logtailer_logfile_change', args=(object_id, )))
        return response


class FilterAdmin(admin.ModelAdmin):
    list_display = ('name', 'regex')


class LogsClipboardAdmin(admin.ModelAdmin):
    list_display = ('name', 'notes', 'log_file')
    readonly_fields = ('name', 'notes', 'logs', 'log_file')


admin.site.register(LogFile, LogFileAdmin)
admin.site.register(Filter, FilterAdmin)
admin.site.register(LogsClipboard, LogsClipboardAdmin)
